@@ -43,9 +43,7 @@ combined_features = FeatureUnion([("pca", pca), ("univ_select", selection)])
 # Use combined features to transform dataset:
 X_features = combined_features.fit(X, y).transform(X)
 
-# Classify:
 svm = SVC(kernel="linear")
-svm.fit(X_features, y)
 
 # Do grid search over k, n_components and C:
 
