@@ -41,7 +41,8 @@ from .externals.six.moves import xrange
 from .utils import check_random_state
 from .utils.extmath import safe_sparse_dot
 from .utils.random import sample_without_replacement
-from .utils.validation import check_arrays
+from .utils.validation import check_array, NotFittedError
+from .utils import DataDimensionalityWarning
 
 
 __all__ = ["SparseRandomProjection",
@@ -74,6 +75,8 @@ def johnson_lindenstrauss_min_dim(n_samples, eps=0.1):
     the larger the dataset, the higher is the minimal dimensionality of
     an eps-embedding.
 
+    Read more in the :ref:`User Guide <johnson_lindenstrauss>`.
+
     Parameters
     ----------
     n_samples : int or numpy array of int greater than 0,
@@ -157,6 +160,8 @@ def gaussian_random_matrix(n_components, n_features, random_state=None):
 
         N(0, 1.0 / n_components).
 
+    Read more in the :ref:`User Guide <gaussian_random_matrix>`.
+
     Parameters
     ----------
     n_components : int,
@@ -202,6 +207,8 @@ def sparse_random_matrix(n_components, n_features, density='auto',
       -  0                              with probability 1 - 1 / s
       - +sqrt(s) / sqrt(n_components)   with probability 1 / 2s
 
+    Read more in the :ref:`User Guide <sparse_random_matrix>`.
+
     Parameters
     ----------
     n_components : int,
@@ -334,10 +341,7 @@ class BaseRandomProjection(six.with_metaclass(ABCMeta, BaseEstimator,
         self
 
         """
-        X, y = check_arrays(X, y)
-
-        if not sp.issparse(X):
-            X = np.atleast_2d(X)
+        X = check_array(X, accept_sparse=['csr', 'csc'])
 
         n_samples, n_features = X.shape
 
@@ -367,7 +371,8 @@ class BaseRandomProjection(six.with_metaclass(ABCMeta, BaseEstimator,
                     "The number of components is higher than the number of"
                     " features: n_features < n_components (%s < %s)."
                     "The dimensionality of the problem will not be reduced."
-                    % (n_features, self.n_components))
+                    % (n_features, self.n_components),
+                    DataDimensionalityWarning)
 
             self.n_components_ = self.n_components
 
@@ -400,10 +405,10 @@ class BaseRandomProjection(six.with_metaclass(ABCMeta, BaseEstimator,
             Projected array.
 
         """
-        X, y = check_arrays(X, y)
+        X = check_array(X, accept_sparse=['csr', 'csc'])
 
         if self.components_ is None:
-            raise ValueError('No random projection matrix had been fit.')
+            raise NotFittedError('No random projection matrix had been fit.')
 
         if X.shape[1] != self.components_.shape[1]:
             raise ValueError(
@@ -411,9 +416,6 @@ class BaseRandomProjection(six.with_metaclass(ABCMeta, BaseEstimator,
                 'X at fit stage had a different number of features. '
                 '(%s != %s)' % (X.shape[1], self.components_.shape[1]))
 
-        if not sp.issparse(X):
-            X = np.atleast_2d(X)
-
         X_new = safe_sparse_dot(X, self.components_.T,
                                 dense_output=self.dense_output)
         return X_new
@@ -424,6 +426,8 @@ class GaussianRandomProjection(BaseRandomProjection):
 
     The components of the random matrix are drawn from N(0, 1 / n_components).
 
+    Read more in the :ref:`User Guide <gaussian_random_matrix>`.
+
     Parameters
     ----------
     n_components : int or 'auto', optional (default = 'auto')
@@ -452,10 +456,10 @@ class GaussianRandomProjection(BaseRandomProjection):
 
     Attributes
     ----------
-    ``n_component_`` : int
+    n_component_ : int
         Concrete number of components computed when n_components="auto".
 
-    ``components_`` : numpy array of shape [n_components, n_features]
+    components_ : numpy array of shape [n_components, n_features]
         Random matrix used for the projection.
 
     See Also
@@ -508,6 +512,8 @@ class SparseRandomProjection(BaseRandomProjection):
       -  0                              with probability 1 - 1 / s
       - +sqrt(s) / sqrt(n_components)   with probability 1 / 2s
 
+    Read more in the :ref:`User Guide <sparse_random_matrix>`.
+
     Parameters
     ----------
     n_components : int or 'auto', optional (default = 'auto')
@@ -556,13 +562,13 @@ class SparseRandomProjection(BaseRandomProjection):
 
     Attributes
     ----------
-    ``n_component_`` : int
+    n_component_ : int
         Concrete number of components computed when n_components="auto".
 
-    ``components_`` : CSR matrix with shape [n_components, n_features]
+    components_ : CSR matrix with shape [n_components, n_features]
         Random matrix used for the projection.
 
-    ``density_`` : float in range 0.0 - 1.0
+    density_ : float in range 0.0 - 1.0
         Concrete density computed from when density = "auto".
 
     See Also
